From 6b7140e749c7ebd0d6f38825a31e55fef06c68cf Mon Sep 17 00:00:00 2001
From: Sebastian Poeplau <sebastian.poeplau@eurecom.fr>
Date: Mon, 5 Nov 2018 10:40:19 +0100
Subject: [PATCH 1/2] Beginnings of a small tool to count instructions after
 lifting

---
 tools/CMakeLists.txt                |  1 +
 tools/liftandcount/CMakeLists.txt   | 12 ++++++
 tools/liftandcount/LiftAndCount.cpp | 58 +++++++++++++++++++++++++++++
 3 files changed, 71 insertions(+)
 create mode 100644 tools/liftandcount/CMakeLists.txt
 create mode 100644 tools/liftandcount/LiftAndCount.cpp

diff --git a/tools/CMakeLists.txt b/tools/CMakeLists.txt
index 4c99358..7e36dce 100644
--- a/tools/CMakeLists.txt
+++ b/tools/CMakeLists.txt
@@ -27,6 +27,7 @@ add_subdirectory(forkprofiler)
 add_subdirectory(hardening)
 add_subdirectory(icounter)
 add_subdirectory(lcoverage)
+add_subdirectory(liftandcount)
 add_subdirectory(pfprofiler)
 add_subdirectory(revgen32)
 add_subdirectory(revgen64)
diff --git a/tools/liftandcount/CMakeLists.txt b/tools/liftandcount/CMakeLists.txt
new file mode 100644
index 0000000..a8bbadf
--- /dev/null
+++ b/tools/liftandcount/CMakeLists.txt
@@ -0,0 +1,12 @@
+add_executable(liftandcount LiftAndCount.cpp)
+
+target_link_libraries(liftandcount
+  X8664Translator
+  utils
+  vmi
+  bsd
+  elf
+  ${LLVM_LIBS}
+  ${GLIB2_LIBRARIES})
+
+install(TARGETS liftandcount RUNTIME DESTINATION bin)
diff --git a/tools/liftandcount/LiftAndCount.cpp b/tools/liftandcount/LiftAndCount.cpp
new file mode 100644
index 0000000..8251f6f
--- /dev/null
+++ b/tools/liftandcount/LiftAndCount.cpp
@@ -0,0 +1,58 @@
+#include <iostream>
+#include <fstream>
+#include <string>
+
+#include <Translator/Translator.h>
+#include <vmi/ExecutableFile.h>
+#include <vmi/FileProvider.h>
+
+
+int main(int argc, char *argv[])
+{
+    if (argc != 4) {
+        std::cerr << "Usage: " << argv[0] << " bitcodeLibrary binary addressFile"
+                  << std::endl;
+        return -1;
+    }
+
+    std::string bitcodeLibraryName = argv[1],
+        binaryName = argv[2],
+        addressFileName = argv[3];
+
+    auto binaryProvider = vmi::FileSystemFileProvider::get(binaryName, false);
+    if (!binaryProvider) {
+        std::cerr << "Failed to open " << binaryName << std::endl;
+        return -1;
+    }
+
+    auto binary = vmi::ExecutableFile::get(binaryProvider, false, 0);
+    if (!binary) {
+        std::cerr << "Failed to parse " << binaryName << std::endl;
+        return -1;
+    }
+
+    std::ifstream addressFile(addressFileName);
+    while (!addressFile.eof()) {
+        uint64_t start, end;
+        addressFile >> std::hex >> start >> end;
+        if (!addressFile.good())
+            break;
+
+        auto translator = s2etools::X86Translator(bitcodeLibraryName, binary);
+        auto translatedBlock = translator.translate(start, end);
+        auto llvmFunction = translatedBlock->getFunction();
+        if (llvmFunction->size() != 1) {
+            std::cerr << "Error: translation yielded more than one basic block" << std::endl;
+            return -1;
+        }
+
+        // for (auto& inst : llvmFunction->front()) {
+        //     std::cout << inst.getName().str() << std::endl;
+        // }
+        llvmFunction->dump();
+        std::cout << llvmFunction->front().size() << std::endl;
+    }
+
+    std::cout << "Done" << std::endl;
+    return 0;
+}
-- 
2.23.0

